@@ -12,7 +12,7 @@ from pyraptor.model.structures import (
     Label,
     Leg,
     Journey,
-    pareto_set_labels,
+    pareto_set,
 )
 from pyraptor.util import TRANSFER_COST
 
@@ -124,19 +124,17 @@ class McRaptorAlgorithm:
                 # Step 1: update earliest arrival times and criteria for each label L in route-bag
                 update_labels = []
                 for label in route_bag.labels:
-                    if label.trip is not None:
-                        trip_stop_time = label.trip.get_stop(current_stop)
+                    trip_stop_time = label.trip.get_stop(current_stop)
 
-                        # Take fare of previous stop in trip as fare is defined on start
-                        previous_stop = remaining_stops_in_route[stop_idx - 1]
-                        from_fare = label.trip.get_fare(previous_stop)
+                    # Take fare of previous stop in trip as fare is defined on start
+                    previous_stop = remaining_stops_in_route[stop_idx - 1]
+                    from_fare = label.trip.get_fare(previous_stop)
+
+                    label = label.update(
+                        earliest_arrival_time=trip_stop_time.dts_arr,
+                        fare_addition=from_fare,
+                    )
 
-                        label = label.update(
-                            earliest_arrival_time=trip_stop_time.dts_arr,
-                            fare_addition=from_fare,
-                        )
-                    else:
-                        label = label.update()
                     update_labels.append(label)
                 route_bag = Bag(labels=update_labels)
 
@@ -150,14 +148,13 @@ class McRaptorAlgorithm:
 
                 # Mark stop if bag is updated
                 if bag_update:
-                    if current_stop != marked_stop:
-                        new_marked_stops.add(current_stop)
+                    new_marked_stops.add(current_stop)
 
                 # Step 3: merge B_{k-1}(p) into B_r
                 route_bag = route_bag.merge(bag_round_stop[k - 1][current_stop])
 
                 # Assign trips to all newly added labels in route_bag
-                # This is the trip on which we 'hop-on'
+                # This is the trip on which we board
                 update_labels = []
                 for label in route_bag.labels:
                     earliest_trip = marked_route.earliest_trip(
@@ -165,7 +162,7 @@ class McRaptorAlgorithm:
                     )
                     if earliest_trip is not None:
                         # Update label with earliest trip in route leaving from this station
-                        # If trip is different we hop-on the trip at current_stop
+                        # If trip is different we board the trip at current_stop
                         label = label.update_trip(earliest_trip, current_stop)
                         update_labels.append(label)
                 route_bag = Bag(labels=update_labels)
@@ -243,7 +240,7 @@ def best_legs_to_destination_station(
 
     # TODO Use merge function on Bag
     # Pareto optimal labels
-    pareto_optimal_labels = pareto_set_labels([label for (_, label) in best_labels])
+    pareto_optimal_labels = pareto_set([label for (_, label) in best_labels])
     pareto_optimal_labels = [
         (stop, label) for (stop, label) in best_labels if label in pareto_optimal_labels
     ]
@@ -313,30 +310,3 @@ def reconstruct_journeys(
     journeys = [jrny for jrny in loop(bag_round_stop, k, journeys)]
 
     return journeys
-
-
-def pareto_optimal_journeys(journeys: List[Journey]) -> List[Journey]:
-    """
-    Select all pareto optimal journeys from a list of journeys.
-
-    Example:
-        Given
-            Journey from 5:00 to 7:00, fare €0
-            Journey from 5:30 to 7:30, fare €0
-            Journey from 5:30 to 6:30, fare €5
-            Journey from 5:30 to 7:00, fare €0
-        we expect the first and second journey to be removed as they are
-        dominated by the last journey.
-    """
-    best_journeys = list()
-
-    for jrny in journeys:
-        # Check if this journey is dominated by any other journey
-        dominated = [
-            True if other_jrny.dominates(jrny) else False for other_jrny in journeys
-        ]
-        if not any(dominated) and jrny not in best_journeys:
-            best_journeys.append(jrny)
-
-    best_journeys = sorted(best_journeys)
-    return best_journeys
